import scrape
import os
import sys
from appJar import gui
import threading
import time

class Globals:
	KAppName = "RGL history and game info creator"

class PlatformDownloader:
	def __init__(self):
		self._thread = threading.Thread(target=lambda: self._DoRun())

	def IsDone(self):
		return not self._platforms is None
		
	def Run(self):
		self._platforms = None
		self._thread.start()

	def _DoRun(self):
		self._platforms = scrape.Platforms()
	
	def GetList(self):
		return self._platforms.GetList()
		
		
class GamesDownloader:
	def __init__(self):
		self._thread = threading.Thread(target=lambda: self._DoRun())
		self.count = 0
		self.total = 1

	def _Update(self, aCount, aTotal):
		self.count = aCount
		self.total = aTotal

	def _DoRun(self):
		self._games = scrape.CreateOutputFilesForPlatform(self._platform, lambda x, y: self._Update(x, y))
		self._done = True
	
	def IsDone(self):
		return self._done
	
	def Run(self, aPlatform):
		self._done = False
		self._platform = aPlatform
		self._thread.start()

	def GetPlatform(self):
		return self._platform

		
class GenerateInfoGUI:
	def _PollGameDownLoads(self):
		gamesDownloaders = self._gamesDownloaders
		for x in gamesDownloaders:
			if (not x.IsDone()):
				p = (100 * x.count) / x.total
				self._app.setMeter(x.GetPlatform().name, p)
			else:
				self._app.setMeter(x.GetPlatform().name, 100, "Finished")
				self._CompletedDownload(x)

	def _CompletedDownload(self, aGamesDownloader):
		self._gamesDownloaders = [x for x in self._gamesDownloaders if not x.GetPlatform().id is aGamesDownloader.GetPlatform().id]
		if (len(self._gamesDownloaders) == 0):
			self._app.setStatusbar("All done", 0)
		
	def _StartDownloads(self):
		self._gamesDownloaders = []
		for x in self._platforms:
			gamesDownloader = GamesDownloader()
			gamesDownloader.Run(x)
			self._gamesDownloaders.append(gamesDownloader)
	
	def Run(self, aPlatforms):
		self._platforms = aPlatforms
		
		self._app = gui(Globals.KAppName)
		self._app.setPadding([0, 0])
		
		self._app.startLabelFrame("Platforms")
		#self._app.setSticky("ew")
		for x in aPlatforms:
			self._app.addLabel("label"+x.name, x.description, self._app.gr(), 0)
			self._app.addMeter(x.name, self._app.gr()-1, 1)
			#App.app.setMeter(x, 0)
		self._app.stopLabelFrame()
		
		self._app.addStatusbar(fields=1)
		self._app.setStatusbar("Downloading...", 0)

		self._app.registerEvent(lambda: self._PollGameDownLoads())

		self._StartDownloads()
		
		self._app.go()
	
		
class SelectPlatformsGUI:
	KPlatformListBox = "Platforms"
	
	def __init__(self):
		self._platformDownloader = PlatformDownloader()

	def GenerateButtonPressed(self):
		self._selectedPlatformList = self._app.getListItems(SelectPlatformsGUI.KPlatformListBox)
		self._app.stop()
	
	def PollPlatformDownLoads(self):
		if (not self._platformDownloader is None):
			if (self._platformDownloader.IsDone()):
				self.DownloadPlatformDone(self._platformDownloader.GetList())
				self._platformDownloader = None

	def DownloadPlatformDone(self, aList):
		self._fullPlatformList = aList
		listboxList = sorted([x.description for x in aList])
		self._app.updateListItems(SelectPlatformsGUI.KPlatformListBox, listboxList)
		if (len(listboxList) > 0):
			self._app.selectListItem(SelectPlatformsGUI.KPlatformListBox, listboxList[0])
		self._app.setButtonState("Generate", "normal")
		self._app.setStatusbar("Done", 0)

	def Run(self):
		self._app = gui(Globals.KAppName, "800x600")

		self._app.startLabelFrame(SelectPlatformsGUI.KPlatformListBox)
		self._app.setSticky("ew")

		self._app.addListBox("Platforms", [])
		self._app.setListBoxMulti(SelectPlatformsGUI.KPlatformListBox)
		self._app.stopLabelFrame()

		self._app.addButton("Generate",  lambda x: self.GenerateButtonPressed())
		self._app.setButtonState("Generate", "disabled")

		self._app.addStatusbar(fields=1)
		self._app.setStatusbar("Please wait, downloading platform list", 0)
		
		self._app.setListBoxHeight(SelectPlatformsGUI.KPlatformListBox, 25)
		self._app.setListBoxInPadding(SelectPlatformsGUI.KPlatformListBox, [50, 50])

		self._app.registerEvent(lambda: self.PollPlatformDownLoads())

		self._selectedPlatformList = []
		self._platformDownloader.Run()
		self._app.go()
		
		return [x for x in self._fullPlatformList if x.description in self._selectedPlatformList]
	
selectPlatformsGUI = SelectPlatformsGUI()
list = selectPlatformsGUI.Run()

if (len(list) > 0):
	generateInfoGUI = GenerateInfoGUI()
	generateInfoGUI.Run(list)

# sys.exit waits for the background threads to finish - os._exit does not
os._exit(0)
