/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "favsgames.h"
#include "fileutils.h"
#include "sdlmenu2dmodel.h"
#include "appcommands.h"
#include "sdlinfoprint.h"
#include "userproperties.h"
#include "systemproperties.h"
#include "appmenudefs.h"
#include "settingsdialogs.h"
#include "controllerutils.h"


using namespace FavsGames;

std::string Model::KFileName("favs.rc");

void Model::Construct()
{
	TStringPairList stringPairList;
	FileUtils::ReadFile(AppSystemProperties.iUserDir + Platform::PathSeperator() + KFileName, stringPairList);
	iMenu = sdl::Menu2DModel::ConstructMenu(KMenuNameFavorites, KMenuTypeFavs, stringPairList);
	iMenu->Sort();
	Reconfigure();
}

void Model::Reconfigure()
{
	if (iMenu)
	{
		iMenu->SetEnabled(AppUserProperties.iFavsGames.iEnabled);
		iMenu->PruneMenuItems(AppUserProperties.iFavsGames.iMax);
	}
}

void Model::Add(const sdl::MenuItem& aMenuItem)
{
	if (iMenu)
	{
		if (!iMenu->HasMenuItem(aMenuItem.GetCommand()))
		{
			if (iMenu->MenuSize() < AppUserProperties.iFavsGames.iMax)
			{
				// add, sort and write
				iMenu->AddMenuItem(
					new sdl::MenuItem(aMenuItem.GetName().c_str(), aMenuItem.GetCommand().c_str()),
					sdl::Menu::TAddPolicy::EAddToBack,
					sdl::Menu::TNotifiyPolicy::EDontNotifyClients
					);
				iMenu->Sort();
				Persist();
				sdl::InfoPrint::Message("Game added to favorites");
			}
			else
				sdl::InfoPrint::Message("Game not added - favorites list is full");
		}
		else
			sdl::InfoPrint::Message("Game already in favorites list");
	}
}

void Model::Delete(const sdl::MenuItem& aMenuItem)
{
	if (iMenu)
	{
		if (sdl::StdDialogs::MessageDialog("Delete game from favorites", "Are you sure?"))
		{
			// delete and write
			iMenu->DeleteMenuItem(aMenuItem.GetCommand());
			Persist();
		}
	}
}

void Model::Reset()
{
	if (iMenu)
	{
		iMenu->DeleteAllMenuItems();
		Persist();
	}
}


void Model::Persist()
{
	if (iMenu)
	{
		TStringPairList stringPairList;
		sdl::Menu2DModel::GetMenuList(*iMenu, stringPairList);
		FileUtils::WriteFile(AppSystemProperties.iUserDir + Platform::PathSeperator() + KFileName, stringPairList);
	}
}

void FavsGamesMenuController::MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand)
{
	if ((aCommand == (int) EAddtoFavorites) && !aMenuItem.GetCommand().empty() && aMenuItem.GetName() != KMenuNameFavorites)
	{
		// only add to favs from menus with games (i.e. !aMenuItem.first.empty) and menus other than the Favs menu
		iModel.Add(aMenuItem);
	}
	else if ((aCommand == sdl::EDelete) && (aMenuItem.GetParentMenu().GetName() == KMenuNameFavorites))
	{
		// only delete from Favs sdl::Menu
		iModel.Delete(aMenuItem);
		sdl::InfoPrint::Message("Game deleted from favorites");
	}
}

void SettingsMenuController::MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand)
{
	if (aCommand == sdl::ESelect)
	{
		if (aMenuItem.GetName().compare(KMenuCmdClearFavs) == 0)
			CmdClearFavs();
		else if (aMenuItem.GetName().compare(KMenuCmdConfigureFavsMenu) == 0)
			CmdConfigureFavsMenu();
	}
}

void SettingsMenuController::CmdClearFavs()
{
	if (sdl::StdDialogs::MessageDialog("Clearing favorite games", "Are you sure?"))
	{
		iModel.Reset();
		sdl::InfoPrint::Message("Favorite games cleared");
	}
}

void SettingsMenuController::CmdConfigureFavsMenu()
{
	DialogConfigureSystemMenu dialogSettingsFavsMenu(AppUserProperties.iFavsGames.iEnabled, AppUserProperties.iFavsGames.iMax, std::string("favorites"));
	dialogSettingsFavsMenu.SetTitle(KMenuCmdConfigureFavsMenu);
	if (dialogSettingsFavsMenu.Execute())
	{
		ControllerUtils::SaveResourceFileUserProperties();		
		iModel.Reconfigure();
	}
}

