/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_DECORATOR_H
#define __GAME_DECORATOR_H

#include "sdlimenulistener.h"
#include "sdlcontainer.h"
#include "sdlimagewidget.h"
#include "sdlsimpletextwidget.h"
#include "vlcwidget.h"
#include "themeproperties.h"
#include "game.h"

class GameEngineManager;

class GameDecorator : public sdl::IMenuItemListener, public sdl::IMenuChangedListener, public sdl::Container
{
public:
	GameDecorator(GameEngineManager& aGameEngineManager)  : iTimerId(0), iGameEngineManager(aGameEngineManager) { SetName("GameDecorator");  }
	~GameDecorator();
	void Construct();
	
private: // from IMenuItemListener
	virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override;
	virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override { }
	virtual void MenuListChanged(sdl::Menu& aMenu) override;
	
private: // from IMenuChangedListener	
	virtual void MenuChanged(sdl::Menu& aMenu) override;

private:
	virtual void Render(sdl::IRenderer& aRenderer) override;
	virtual void ResetSDLState() override;
	
	void UpdateGameInfo(const TGame* aGame);

	void UpdateGameImage(sdl::ImageWidget& aImage, const ThemeProperties::GameImageProperties& aGameImageResource, const TGame* aGame);

	void ConstructGameImages();
	void ConstructGameTexts();
	void ConstructGameVideo();
	

	void ConstructCategoryLabel();
	void SetCatergoryText(const std::string& aMenuName, int aNumberOfItems);

	void StartTimer();
	void CloseTimer();
	static Uint32 TimerCallback(Uint32 aInterval, void* aParam);
	Uint32 TimerCallback(Uint32 aInterval);
	void PlayVideo();
	
	sdl::SimpleTextWidget	iCategoryText;

	std::string				iCurrentGameID;
	sdl::SimpleTextWidget 	iText[KNumOfSimpleTexts];
	sdl::ImageWidget		iGameImage[KNumOfGameImages];
	VlcWidget				iVideoWidget;
	
	bool 					iIsGameMenu = true;
	SDL_TimerID				iTimerId;
	bool 					iPlayVideo = false;
	
	GameEngineManager& 		iGameEngineManager;
};

#endif // __GAME_DECORATOR_H