/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_ENGINE_H
#define __GAME_ENGINE_H

#include "types.h"
#include "gameengineproperties.h"
#include "gameinfomodel.h"
#include "mediadirectorymapper.h"

class GameEngine
{
public:
	GameEngine() { }
	virtual ~GameEngine() { }
	
	void Construct(const TGameEngineProperties& aGameEngineProperties, const std::string& aDataDir, const std::string& aUserDir)
	{
		iGameEngineProperties = aGameEngineProperties;
		iDataDir = aDataDir;
		iUserDir = aUserDir;
	}
	
	void LoadGamesData(const std::string& aName, const TStringMap& aCategoryMapping, TGameMap& aGameMap);
	void GetGameInfo(const std::string& aGameName, std::string& aGameInfo);
	std::string GetMediaFileName(const std::string& aGameName, const std::string& aDirectory);
	void PopulateMediaDirectoryMapper(const TMediaDirectory& aMediaDirectory, const TGameMap& aGameMap);
	
private:
	void FillGameMap(const std::string& aName, const TStringMap& aCategoryMapping, TGameMap& aGameMap);
	void FillGameMap_DatFileIsLeading(const std::string& aName, TStringMap& aGameInfo);
	void FillGameMap_RomsAreLeading(const std::string& aName, TStringMap& aGameInfo);
	void RemoveUnsupportedGames(TStringMap& aGameInfo);
	void CreateGameInfo(const std::string& aName, const TStringMap& aCategoryMapping, const TStringMap& aGameInfoList, TGameMap& aMameGameMap);
	std::string GetDescription(const std::string& aFileName, char aIgnoreLeading, char aIgnoreTrailing);

	void RemoveGamesWithoutRoms(TStringMap& aGameInfo);
	void AddRomsNotInGamesDatFile(TStringMap& aGameInfo);
	
	static std::string MapCategory(const std::string& aCategory, const TStringMap& aCategoryMapping);

	GameInfoModel 			iGameInfoModel;
	MediaDirectoryMapper 	iMediaDirectoryMapper;
	TGameEngineProperties	iGameEngineProperties;
	std::string 			iDataDir;
	std::string 			iUserDir;
};

#endif // __GAME_ENGINE_H