/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_INFO_MODEL_H
#define __GAME_INFO_MODEL_H

#include "types.h"
#include "game.h"
#include <istream>

class GameEngine;

class GameInfoModel
{
public:
	void Construct(const std::string& aDataDirectory, const std::string& aUserDirectory, const TGameMap& aGameMap, const std::string& aGameEngine);
	void GetGameInfo(const std::string& aGameName, std::string& aGameInfo);

	struct TGameInfoIndexEntry
	{
		int iOffset;
		int iLength;
	};
	typedef std::map<std::string, TGameInfoIndexEntry> TGameInfoIndex;
private:
	TGameInfoIndex iGameInfoIndex;
	
	void BuildIndexFile(const TGameMap& aGameMap);
	void ReadIndexFile();
	void ReadGame(const TGameInfoIndexEntry& aGameInfoIndexFile);
	void ExtractGameNames(std::string& aValue, std::vector<std::string>& aGameNames);
	void WriteGameInfoIndexEntry(std::ofstream& aFileStream, const std::vector<std::string>& aGameNames, const TGameInfoIndexEntry& aGameInfoIndexFile);
	void CreateSupportedGames(const std::vector<std::string>& aGameNames, std::vector<std::string>& aSupportedGameNames, const TGameMap& aGameMap);
	
	void ReadGameInfo(const TGameInfoIndexEntry& aGameInfoIndexEntry, std::string& aGameInfo);
	
	std::string iInfoFileName;
	std::string iInfoIndexFileName;
};


#endif // __GAME_INFO_MODEL_H