/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAMES_CONTAINER_H
#define __GAMES_CONTAINER_H

#include "game.h"
#include "mediadirectory.h"

class GamesContainer;

class GameFilterConfig
{
public:
	int 	iMaxPlayers = 4;
	int 	iMaxButtons = 16;
	bool 	iSimultaneousPlayers = true;
	
	bool IsGameSupported(const TGame& aGame) const;
};

/*
 * Implements an iterator over a class GamesContainer which will skip unsupported games
 * Thought it was fun to write an interator :-)
 */
class GamesIter_const
{
public:
	const TGame& operator*();
	GamesIter_const& operator++();
	bool equals(const GamesIter_const& aOther) const;
	bool operator==(const GamesIter_const& aOther) const { return iGameId == aOther.iGameId; }
	bool operator!=(const GamesIter_const& aOther) const { return !operator==(aOther); }
	const std::string& GameId() const { return iGameId; }
private:
	GamesIter_const(const GamesContainer& aGamesContainer, bool aBegin); 
	GamesIter_const(const GamesContainer& aGamesContainer, const std::string& aGameName);
	void Begin();
	void End();
	void FindNextSupportedGame(const std::string aGameId);
	const GamesContainer& iGamesContainer;
	friend class GamesContainer;
	std::string iGameId; // uniquely identifies entries in our container
};


class GamesContainer
{
public:
	GamesContainer(const std::string& aName, const GameFilterConfig& aGameFilterConfig) : iName(aName), iGameFilterConfig(aGameFilterConfig) {}
	typedef GamesIter_const const_iterator;

	const_iterator begin() const { return const_iterator(*this, true); }
	const_iterator end() const { return const_iterator(*this, false); }
	const_iterator find(const std::string& aGameId) const { return const_iterator(*this, aGameId); }
	
	int size() const { return (int) iGameMap.size(); }
	const std::string& Name() const { return iName; }
	
protected:
	bool IsGameSupported(const TGame& aGame) const { return iGameFilterConfig.IsGameSupported(aGame); }
	friend GamesIter_const;
	friend class GameEngineManager;
	
	TGameMap iGameMap;
	std::string iName;
	const GameFilterConfig& iGameFilterConfig;
};


#endif // __GAMES_CONTAINER_H
