/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_ENGINE_MANAGER_H
#define __GAME_ENGINE_MANAGER_H

#include "gamescontainer.h"

class GameEngine;

class GameEngineManager
{
public:
	virtual ~GameEngineManager();
	typedef std::map<std::string, GamesContainer> TGamesContainer;
	void Construct();
	
	const TGamesContainer& GetGamesContainer() const { return iGamesContainer; }
	
	void GetGameInfo(const std::string& aGameID, std::string& aGameInfo);
	
	const TGame* GetGame(const std::string& aGameID);
	std::string GetMediaFileName(const TGame& aGame, const std::string& aLogicalDirectory);
	std::string GetMediaFileName(const std::string& aGameName, const std::string& aEngineName, const std::string& aLogicalDirectory);
	std::string GetMediaFileName(const std::string& aGameId, const std::string& aLogicalDirectory);
	
	std::string GetRomFileName(const std::string& aGameId);
	
	void SetMaxPlayers(int aMaxPlayers) { iGameFilterConfig.iMaxPlayers = aMaxPlayers; }
	void SetMaxButtons(int aMaxButtons) { iGameFilterConfig.iMaxButtons = aMaxButtons; }
	void SetSimultaneousPlayers(bool aSimultaneousPlayers) { iGameFilterConfig.iSimultaneousPlayers = aSimultaneousPlayers; }
private:
	GameEngine& GetGameEngine(const std::string& aName) const;
	void AddGameEngine(const std::string& aName, GameEngine* aGameEngine);
	void LoadCategoryMap(const std::string& aEngineName, TStringMap& aCategoryMapping);
	
	typedef std::map<std::string, GameEngine*> TGameEngines;
	
	GameFilterConfig	iGameFilterConfig;
	TGamesContainer 	iGamesContainer;
	TGameEngines		iGameEngines;
};


#endif // __GAME_ENGINE_MANAGER_H
