/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_MENUS_H
#define __GAME_MENUS_H

#include "types.h"

namespace sdl
{
	class Menu2DModel;
	class Menu;
}
class GameEngineManager;


typedef std::map<std::string, sdl::Menu*> TMenuLists;

class GameMenus
{
public:
	GameMenus(sdl::Menu2DModel& aMenuModel, const GameEngineManager& aGameEngineManager) : iMenuModel(aMenuModel), iGameEngineManager(aGameEngineManager) {}
	void Construct();
	
	void UpdateCategories();
	void RePopulate();
	
	const std::vector<std::string>& GetCategoriesList() const { return iCategoryNames; }
	const std::vector<std::string>& GetGameEngineNames() const { return iGameEngineNames; }
private:
	void Populate(TMenuLists& aMenuLists);
	void PopulateNoGamesMenu(TMenuLists& aMenuLists);
	sdl::Menu* FindOrCreateMenu(TMenuLists& aMenuLists, const std::string& aCategory);
	void MapCategory(std::string& aCategory);

	bool ShouldIncludeCategory(const std::string& aMenu);
	bool ShouldIncludeGameEngine(const std::string& aGameEngineName);
	
	sdl::Menu2DModel&			iMenuModel;
	const GameEngineManager&	iGameEngineManager;
	std::vector<std::string>	iCategoryNames;
	std::vector<std::string>	iGameEngineNames;
};


#endif // __GAME_MENUS_H
