/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_RUNNER_H
#define __GAME_RUNNER_H

#include "sdlimenulistener.h"
#include "game.h"
#include "types.h"

class Menu;
class GameEngineManager;

namespace GameRunner
{
	class Model;
	
	class RecentGamesMenuController : public sdl::IMenuItemListener
	{
	public:
		RecentGamesMenuController(Model& aModel) : iModel(aModel) { }
		virtual ~RecentGamesMenuController() {}
		
		private: // from IMenuItemListener
			virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override { }
			virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override;
			virtual void MenuListChanged(sdl::Menu& aMenu) override {}
			
			Model& iModel;
	};
	
	class SettingMenuController : public sdl::IMenuItemListener
	{
	public:
		SettingMenuController(Model& aModel) : iModel(aModel) { }
		virtual ~SettingMenuController() {}
		
		private: // from IMenuItemListener
			virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override { }
			virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override;
			virtual void MenuListChanged(sdl::Menu& aMenu) override {}
			
			void CmdClearRecentGames();
			void CmdConfigureRecentGamesMenu();
			
			Model& iModel;
	};
	
	class Model
	{
	public:
		const static std::string KMenuName;
		
		Model(GameEngineManager& aGameEngineManager) : iGameEngineManager(aGameEngineManager), iRecentGamesMenuController(*this), iSettingMenuController(*this) {} 
		virtual ~Model() {}
		
		void Construct();
		bool WantRunGame();
		bool RunGame();
		
		sdl::Menu* 					GetMenu() 						{ return iMenu; }
		RecentGamesMenuController&	GetRecentGamesMenuController()	{ return iRecentGamesMenuController; }
		SettingMenuController&		GetSettingMenuController()		{ return iSettingMenuController; }
	private:
		friend RecentGamesMenuController;
		friend SettingMenuController;
		
		void Reset();
		void Reconfigure();
		void SetRunGame(const sdl::MenuItem& aMenuItem);
		std::string CreateCommandLine(const std::string& aGameName, const std::string& aEngineName);

		void AddToRecent(const sdl::MenuItem& aMenuItem);
		void Persist();

		sdl::Menu*	iMenu = nullptr; // don't own it
		std::string iGameIDToRun;

		static std::string KFileName;
		GameEngineManager& 			iGameEngineManager;
		RecentGamesMenuController	iRecentGamesMenuController;
		SettingMenuController		iSettingMenuController;
	};


}

#endif // __GAME_RUNNER_H