/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SYSTEM_MENU_CONTROLLER_H
#define __SYSTEM_MENU_CONTROLLER_H

#include "sdlmenu2dmodel.h"
#include "sdlimenulistener.h"
#include "types.h"

class GameMenus;
namespace sdl
{
	class Audio;
}

class SystemMenuController : public sdl::IMenuItemListener
{
public:
	SystemMenuController(GameMenus& aGameMenus, sdl::Audio& aAudio);
	void Construct();
	
private: // from IMenuItemListener
	virtual void MenuItemHighlighted(const sdl::MenuItem& aMenuItem) override { }
	virtual void MenuItemCommand(const sdl::MenuItem& aMenuItem, const sdl::TCommand& aCommand) override;
	virtual void MenuListChanged(sdl::Menu& aMenu) override {}
	
	void CmdFilterCategories();
	void CmdFilterGameEngines();
	void CmdConfigureJoystickNavigation();
	void CmdConfigureJoystickScrollSpeed();
	void CmdConfigureJoystickButtons();
	void CmdAbout();
	void CmdAudioEvents();
	void CmdAudioVolume();
	void CmdVideoSettings();
	
	void SaveResourceFileUserProperties();
	void SaveResourceFileJoystickProperties();
	
	typedef void (SystemMenuController::*HandleMenuItemMethod)();
	typedef std::map <std::string, HandleMenuItemMethod> THandleMenuItemMethods;
	THandleMenuItemMethods iHandleMenuItemMethods;
	
	GameMenus&		iGameMenus;
	sdl::Audio&		iAudio;
};


#endif // __SYSTEM_MENU_CONTROLLER_H