/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SYSTEM_PROPERTIES_H
#define __SYSTEM_PROPERTIES_H

#include "platform.h"
#include "gameengineproperties.h"
#include "fileutils.h"
#include <vector>
#include <map>

class SystemProperties;
extern SystemProperties AppSystemProperties;


class SystemProperties
{
public:
	SystemProperties() { SetDefaults(); }
	std::string iDataDir;
	std::string iUserDir;
	std::string iThemeDir;
	
	std::vector<std::string> iFontDirs;
	std::vector<std::string> iImageDirs;
	std::vector<std::string> iAudioDirs;
	
	std::map<std::string, TGameEngineProperties> iGameEnginesProperties;
	
	void SetDefaults()
	{
		iDataDir	= "data";
		iUserDir	= "user";
		iThemeDir	= "theme";
		iFontDirs	= {".", "fonts"};
		iImageDirs	= {".", "images"};
		iAudioDirs	= {".", "sounds"};
		iGameEnginesProperties.clear();
		
		PopulateGameEngines();
	}
private:
	void PopulateGameEngines()
	{
		TStringList subDirs;
		FileUtils::ReadDirectory(iDataDir, subDirs);
		
		for (auto &subDir : subDirs)
		{
			iGameEnginesProperties[subDir] = { "", "", "zip", false };
		}
	}
};


SystemProperties DECLARE_ONCE AppSystemProperties;

#endif // __SYSTEM_PROPERTIES_H