/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "thememodel.h"
#include "systemproperties.h"
#include "userproperties.h"
#include "platform.h"
#include "fileutils.h"
#include "string.h"

static std::string sThemeName;

static bool IsSubDir(dirent* aDirEntry, std::string& aOutName)
{
	if (aDirEntry->d_type == DT_DIR)
	{
		int len = strlen(aDirEntry->d_name);
		if ((len == 1) && (aDirEntry->d_name[0] == '.'))
			return false;
		if ((len == 2) && (aDirEntry->d_name[0] == '.') && (aDirEntry->d_name[1] == '.'))
			return false;
		aOutName.assign(aDirEntry->d_name);
		return true;
	}
	return false;
}

std::string ThemeModel::GetCurrentThemeName()
{
	return AppUserProperties.iThemeName;
}

std::string ThemeModel::GetCurrentThemeDirName()
{
	return AppSystemProperties.iThemeDir + Platform::PathSeperator() + AppUserProperties.iThemeName;
}

std::string ThemeModel::GeThemeDirName(const std::string& aThemeName)
{
	return AppSystemProperties.iThemeDir + Platform::PathSeperator() + aThemeName;
}

void ThemeModel::GetThemeList(TStringList& aStringList)
{
	aStringList.clear();
	
	FileUtils::ReadDirectory("theme", aStringList, IsSubDir);
	std::sort(aStringList.begin(), aStringList.end());
}

void ThemeModel::ChangeTheme(const std::string& aThemeName)
{
	sThemeName = aThemeName;
	AppUserProperties.iThemeName = aThemeName;
	iThemeChanged = true;
}

bool ThemeModel::TestAndResetHasThemeChangedLastRun(std::string& aThemeName)
{
	bool changed = sThemeName.size() > 0;
	aThemeName = sThemeName;
	sThemeName.clear();
	return changed;
}
