/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __THEME_PROPERTIES_H
#define __THEME_PROPERTIES_H

#include "platform.h"

#include "sdlthemeproperties.h"
#include "sdlimagewidget.h"
#include "sdlsimpletextwidget.h"
#include "systemproperties.h"

class ThemeProperties;
extern ThemeProperties AppThemeProperties;

const int KNumOfSimpleTexts = 10;
const int KNumOfGameImages = 10;

class ThemeProperties
{
public:
	ThemeProperties() { SetDefaults(); }
	
	class GameInfoDialogProperties
	{
	public:
		int			iWidth;
		int			iHeight;
		std::string	iFormatTitle;
		std::string	iFormatHeading;
		std::string	iFormatBody;
	};

	class GameImageProperties : public sdl::ImageWidget::Properties
	{
	public:
		std::string	iFileName;
		std::string	iDefaultImage;
		std::string	iLogicalDirectory;
	};
	
	class GameVideoProperties
	{
	public:
		sdl::TRect iRect;
		std::string	iLogicalDirectory;
	};
	
	class SimpleTextProperties
	{
	public:
		SimpleTextProperties(const std::string& aText="") : iText(aText) { iProperties.iEclipseText = true;    iProperties.iSingleLine = true; } 
		sdl::TRect							iRect;
		std::string							iText;
		sdl::SimpleTextWidget::Properties	iProperties;
	};

	std::string 				iBackgroundImage;
	GameInfoDialogProperties	iGameInfoDialogProperties;
	SimpleTextProperties		iGameSimpleText[KNumOfSimpleTexts];
	SimpleTextProperties		iCategoryTextProperties;
	std::string					iSystemMenuCategoryText;
	std::string					iExtraAboutText;
	GameImageProperties			iGameImageProperties[KNumOfGameImages];
	GameVideoProperties			iGameVideoProperties;
	
	void SetDefaults()
	{
		iBackgroundImage = "";
		iGameInfoDialogProperties = {-1, -1, "<p align=center><font size=xlarge><b>", "<p align=center><font size=large><b>", "<p align=justified><font size=xxsmall>"};
		iCategoryTextProperties = SimpleTextProperties("<p align=center><font size=large>%cat% (%numitems%)");
		iSystemMenuCategoryText = "<p align=center><font size=large>";
		
		for (int i=0 ; i<KNumOfSimpleTexts ; i++)
			iGameSimpleText[i] = SimpleTextProperties();

		for (int i=0 ; i<KNumOfGameImages ; i++)
			iGameImageProperties[i] = GameImageProperties();
		
		iGameVideoProperties = GameVideoProperties();
	}
};

ThemeProperties DECLARE_ONCE AppThemeProperties;

#endif // __THEME_PROPERTIES_H