/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "resourcefile.h"
#include "fileutils.h"
#include "stringutils.h"

#include <stdio.h>

/****************************
 * 
 * Resource file save/load
 * 
 ***************************/

class TResourceFile
{
public:
	static void ParseResourceFile(Resources& aResources, std::ifstream& aFile, const std::string& aFileName)
	{
		std::string line;
		int lineNum = 0;
		while( std::getline(aFile, line) )
		{
			lineNum++;
			
			if (line.size() == 0)
				continue;
			if (line[0] == '#')
				continue;
			
			std::istringstream is_line(line);
			std::string key;
			
			std::getline(is_line, key, '=');

			StringUtils::StripWhiteSpaces(key);
			
			std::string value;
			std::getline(is_line, value);
			
			StringUtils::StripWhiteSpaces(value);
			if (value.size() > 0)
			{
				
				bool keyNotFound;
				if (!aResources.Decode(key, value, keyNotFound))
				{
					printf("Warning: in resource file \"%s\" (line %d), the line \"%s\" is ignored because %s\n", 
							aFileName.c_str(), lineNum, line.c_str(), keyNotFound ? "the key is not recognised" : "the value was incorrect");
				}
			}
			else
			{
				printf("Warning: in resource file \"%s\" (line %d), the line \"%s\" is ignored\n", aFileName.c_str(), lineNum, line.c_str());
			}
		}
	}

	static void SaveResourceFile(const Resources& aResources, std::ofstream& aFile, const std::string& aFileName)
	{
		std::vector<std::string> resourceNames;
		std::vector<std::string> encodedStrings;
		aResources.Encode(resourceNames, encodedStrings);

		int count = resourceNames.size();
		for (int i=0 ; i<count ; i++)
			aFile << resourceNames.at(i) << " = " << encodedStrings.at(i) << std::endl;
	}
};

bool ResourceFile::Load(const std::string& aFileName, const std::string& aDirectoryName)
{
	bool success = false;
	FileUtils::FindFile(aFileName, aDirectoryName, iFileName);
	
	if (iFileName.length() > 0)
	{
		std::ifstream file;
		file.open(iFileName.c_str());
		if (file.good())
		{
			TResourceFile::ParseResourceFile(iResources, file, iFileName);
			success = true;
			file.close();
			iResources.ApplyDefaults();
			OnLoad();
		}
		else
		{
			printf("ParseResourceFile, could not open found resource file=%s, ignoring it\n", iFileName.c_str());
		}
	}
	else
	{
		printf("ParseResourceFile, could not find any instance of resource file=%s, ignoring it\n", aFileName.c_str());
	}
	return success;
}


bool ResourceFile::Save()
{
	bool success = false;
	std::ofstream file;
	file.open(iFileName.c_str(), std::ofstream::out | std::ofstream::trunc);
	if (file.good())
	{
		file << "#\n";
		file << "# This is an autogenerated file - do not edit\n";
		file << "#\n";
		TResourceFile::SaveResourceFile(iResources, file, iFileName);
		file.close();
		success = true;
	}
	else
	{
		printf("SaveResourceFile, could not find save resource file=%s, ignoring it\n", iFileName.c_str());
	}
	return success;
}
