/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "resources.h"
#include "stringutils.h"



/****************************
 * 
 * Encoders
 * 
 ***************************/

template<>
void Encode(std::string& aEncodedData, std::string& aRawResourceItem)
{
	aEncodedData = "\"" + aRawResourceItem + "\"";
}

template<>
void Encode(std::string& aEncodedData, int& aRawResourceItem)
{
	aEncodedData = std::to_string(aRawResourceItem);
}

template<>
void Encode(std::string& aEncodedData, bool& aRawResourceItem)
{
	aEncodedData.assign(aRawResourceItem ? "true" : "false");
}

template<>
void Encode(std::string& aEncodedData, float& aRawResourceItem)
{
	aEncodedData = std::to_string(aRawResourceItem);
}

template<>
void Encode(std::string& aEncodedData, std::vector<std::string>& aRawResourceItem)
{
	aEncodedData = "";
	bool first = true;
	for (auto &s: aRawResourceItem)
	{
		if (first)
		{
			aEncodedData = "\"" + s + "\"";
			first = false;
		}
		else
		{
			aEncodedData += ", \"" + s + "\"";
		}
	}
}


/****************************
 * 
 * Decoders
 * 
 ***************************/

template<>
bool Decode(const std::string& aEncodedData, std::string& aRawResourceItem)
{
	aRawResourceItem = aEncodedData;
	bool success = StringUtils::StripQuotes(aRawResourceItem);
	return success;
}

template<>
bool Decode(const std::string& aEncodedData, int& aRawResourceItem)
{
	bool success = true;
	size_t pos = aEncodedData.find("0x");
	if (pos != std::string::npos)
	{
		char* p;
		aRawResourceItem = strtol( aEncodedData.c_str() + pos, &p, 16);
		if (*p != 0)
		{
			aRawResourceItem = 0;
			success = false;
		}
	}
	else
	{
		aRawResourceItem = atoi(aEncodedData.c_str());
	}
	return success;
}

template<>
bool Decode(const std::string& aEncodedData, bool& aRawResourceItem)
{
	return StringUtils::GetBoolValueFromString(aEncodedData, aRawResourceItem);
}

template<>
bool Decode(const std::string& aEncodedData, float& aRawResourceItem)
{
	aRawResourceItem = (float) atof(aEncodedData.c_str());
	return true;
}

template<>
bool Decode(const std::string& aEncodedData, std::vector<std::string>& aRawResourceItem)
{
	StringUtils::SplitString(aEncodedData, aRawResourceItem);
	for (auto &s: aRawResourceItem)
	{
		StringUtils::StripQuotes(s);
		StringUtils::StripWhiteSpaces(s);
	}
	return true;
}

/****************************
 * 
 * Resources
 * 
 ***************************/

bool Resources::Decode(const std::string& aKey, const std::string& aValue, bool& aKeyNotFound)
{
	bool success = true;
	aKeyNotFound = false;
	auto it = iResources.find(aKey);
	if (it != iResources.end())
	{
		ResourceItemBase* resourceItem = it->second.get();
		success = resourceItem->Decode(aValue);
	}
	else
	{
		success = false;
		aKeyNotFound = true;
	}
	return success;
}

void Resources::Encode(std::vector<std::string>& aResourceNames, std::vector<std::string>& aEncodedStrings) const
{
	for (auto &resource : iResources)
	{
		aResourceNames.push_back(resource.first);
		std::string encodedString;
		resource.second.get()->Encode(encodedString);
		aEncodedStrings.push_back(encodedString);
	}
}
