/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "renderfactory.h"
#include "sdlrenderer.h"
#include "sdlgpurenderer.h"

using namespace sdl;

typedef IRenderer* (*CreateRendererFactoryMethod)(const TSize&, const std::string&, bool);
std::map <std::string, CreateRendererFactoryMethod> KCreateRendererFactoryMethodMap =
{
	{"SDL2", &CreateSDL2Renderer},
	{"SDLGPU", &CreateSDLGPURenderer}
};


IRenderer* RenderFactory::CreateRenderer(const std::string& aRenderer, const TSize& aSize, const std::string& aWindowTitle, bool aFullScreen)
{
	auto it = KCreateRendererFactoryMethodMap.find(aRenderer);
	if (it != KCreateRendererFactoryMethodMap.end())
	{
		printf("RenderFactory::CreateRenderer, renderer=%s, size(%d, %d), title=%s, fullscreen=%s\n", 
			   aRenderer.c_str(), (int)aSize.iWidth, (int)aSize.iHeight, aWindowTitle.c_str(), aFullScreen ? "yes" : "no");
		return (*it->second)(aSize, aWindowTitle, aFullScreen);
	}
	return nullptr;
}
