/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_AUDIO_H
#define __SDL_AUDIO_H

#include "sdluserfeedbackmanager.h"
#include <string>
#include <map>
#include <vector>
#include <memory>
#include <SDL2/SDL_mixer.h>

namespace sdl
{
	class Audio : public UserFeedbackManager::IUserFeedbackHandler
	{
	public:
		enum class TAudioTypes
		{
			EIntro = 0,
			EOk,
			ECancel,
			EScrollH,
			EScrollV,
			EScrollEnd,
			ELast
		};
		
		class Properties
		{
		public:
			std::vector<std::string> 	iEnabledEvents;
			int						 	iVolume;
			int							iSamples;
			bool 						iMute;
		};
		
		class TAudioData
		{
		public:
			std::string iDescription;
			std::string iFileName;
		};
		
		virtual ~Audio() { ReleaseChunks(); }
		void Construct();
		void GetAudioData(std::vector<TAudioData>& aAudioData);
		void Update();
		void SetVolume(int aPercentage);
		
	protected: // implements UserFeedbackManager::IUserFeedbackHandler
		virtual void HandleEvent(const UserFeedbackManager::TEvent& aEvent) override;

	protected:
		Mix_Chunk* LoadChunk(TAudioTypes& aAudioType);
		void LoadChunks();
		void ReleaseChunks();
		
		std::string GetAudioFileName(TAudioTypes& aAudioType);
		
		void Play(TAudioTypes aAudioType);
		bool IsPlaying();
		bool ShouldPlay(TAudioTypes aAudioType);
		bool IsUserConfigured(TAudioTypes aAudioType);
		
		std::map<TAudioTypes, Mix_Chunk*> iChunkMap;
		TAudioTypes iCurrentAudioType = TAudioTypes::ELast;
	};
}

#endif // __SDL_AUDIO_H
