/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_CORE_H
#define __SDL_CORE_H

#include "sdlbase.h"
#include "sdlrendererinterface.h"

namespace sdl
{
	class SystemDirs
	{
	public:
		void AddDirectoryName(const std::string& aDirectoryName) { iSystemDirs.push_back(aDirectoryName); }
		void SetDirectoryNames(const std::vector<std::string>& aDirectoryNames) { iSystemDirs = aDirectoryNames; }
		const std::vector<std::string>& GetDirectoryNames() const { return iSystemDirs; }
		void Reset() { *this = SystemDirs(); }
	private:
		std::vector<std::string> iSystemDirs = {"."};
	};
	
	class System
	{
	public:
		static void RequestReRender();
		static bool IsReRenderRequested();
		
		static void Init();
		static void Shutdown();

		static SystemDirs FontDirs;
		static SystemDirs ImageDirs;
		static SystemDirs AudioDirs;
	};

	class RenderClipper
	{
	public:
		RenderClipper(IRenderer& aRenderer, const TRect& aRect);
		~RenderClipper();
	private:
		void RestoreClip();
		void Clip(const TRect& aScreenRect);
		
		TRect 		iRect;
		IRenderer&	iRenderer;
	};

	class RenderUtils
	{
	public:
		static ITexture* CreateSVGRoundedRectTexture(IRenderer& aRenderer, const TSize& aSize, const TRoundedRect& aRoundedRect);
		static ITexture* CreateSVGDuoColorFilledRoundedRectTexture(IRenderer& aRenderer, const TSize& aSize, const TRoundedRect& aRoundedRect, const TColor& aFillColor, int aPercentage);
		static ITexture* CreateSVGArrowRightTexture(IRenderer& aRenderer, int aHeight, const TColor& aFrameColor, const TColor& aFillColor);
		static ITexture* CreateSVGCircleTexture(IRenderer& aRenderer, const TSize& aSize, int aFrameSize, const TColor& aFrameColor, const TColor& aFillColor);
		static ITexture* CreateTextureFromText(IRenderer& aRenderer, const TFont& aFont, const std::string& aText, int aMaxWidth, bool aEclipseText);

		static ITexture* CreateTextureFromFile(IRenderer& aRenderer, const std::string& aFileName);
		
		static void RenderScaledTextureMaintainAspectRatio(IRenderer& aRenderer, ITexture& aTexture, const TRect& aBoundingRect, TRect& aOutRect);
		static void RenderScaledTextureMaintainAspectRatio(IRenderer& aRenderer, ITexture& aTexture, const TRect& aBoundingRect)
		{
			TRect ignore;
			RenderScaledTextureMaintainAspectRatio(aRenderer, aTexture, aBoundingRect, ignore);
		}
		
		static void RenderRect(IRenderer& aRenderer, const TRect& aRect, const TColor& aColor, int aFrameSize = 1);

		static TSize RenderText(IRenderer& aRenderer, const TPoint& aPoint, const TFont& aFont, const std::string& aText);
		static TSize GetTextSize(const TFont& aFont, const std::string& aText);
		static void FitText(const TFont& aFont, const std::string& aText, std::string& aReturnedText, int aMaxWidth, int& aWidth, int& aHeight, bool aEclipseText);
		static int GetDescent(const TFont& aFont);
	};
	
	
} // namespace sdl

#endif // __SDL_CORE_H