/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_DIALOG_H
#define __SDL_DIALOG_H

#include "sdlcore.h"
#include "sdleventmanager.h"
#include "sdlwindow.h"
#include "sdlbuttonbar.h"
#include "sdllistwidget.h"

namespace sdl
{
	class Dialog : public Window
	{
	public:
		class Properties : public Window::Properties
		{
		public:
			Properties(
				const Window::Properties& aWindowProperties = Window::Properties(),
				const TMargin& aMargin = TMargin(), 
				int aWidth=-1, 
				int aHeight=-1, 
				int aButtonWidth=-1, 
				TColor aBackgroundMask=0x808080A0
			) : Window::Properties(aWindowProperties), iMargin(aMargin), iWidth(aWidth), iHeight(aHeight), iBackgroundMask(aBackgroundMask) {} 
			void OverrideDefaults(const Properties& aOther)
			{
				iMargin.OverrideDefaults(aOther.iMargin);
				OverridePositiveValue(iWidth, aOther.iWidth);
				OverridePositiveValue(iHeight, aOther.iHeight);
				iBackgroundMask.OverrideDefaults(aOther.iBackgroundMask);
			}
			TMargin iMargin;
			int		iWidth;
			int		iHeight;
			TColor	iBackgroundMask;
		};
		void OverrideDefaults(const Properties& aOther) { iDialogProperties.OverrideDefaults(aOther); Window::OverrideDefaults(aOther); ApplyProperties(); }
		
		Dialog();
		
		virtual bool Execute();
		virtual bool HasInput() const override { return true; }
		
	protected:
		virtual void Render(IRenderer& aRenderer) override;
		virtual bool HandleCommandMyself(TCommand aCommand) override;
		
		virtual void Setup() {}
		virtual void OnDialogExit(bool aSuccess) {}
		void CreateFadedBackgroundBackupBitmap();
		void SetDefaultDialogSize();
		
		void SetDefaultProperties();
		void ApplyProperties() { Window::ApplyProperties(); SetInternalMargin(iDialogProperties.iMargin); }
		
		void Done(bool aSuccess) { iEventManager.MarkEventLoopDone(aSuccess); }
		
		Properties iDialogProperties;
		
	private:
		EventManager iEventManager;
		std::unique_ptr<ITexture> iBackupTexture = nullptr;
	};

	
	class SettingsDialog : public Dialog
	{
	public:
		void AddButton(ButtonWidget& aButton, bool aOwnership=false){ iButtonBar.AddButton(aButton, aOwnership); }
		void AddInput(Widget& aWidget, bool aOwnership=false)  		{ iList.AddWidget(aWidget, aOwnership); }
		
		virtual void Layout() override;
		
	protected:
		virtual void Setup() override { AddWidget(iList); AddWidget(iButtonBar); } 
		
		ScrollableContainer	iList;
		ButtonBar			iButtonBar;
	};
	
	
	class StdDialogs
	{
	public:
		static bool MessageDialog(const std::string& aTitle, const std::string& aMessage);
	};
}

#endif // __SDL_DIALOG_H