/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_EVENT_MANAGER_H
#define __SDL_EVENT_MANAGER_H

#include "sdlcore.h"
#include "sdlcommands.h"

namespace sdl 
{
	class Widget;
	
	#define MAKE_KEY_STATE(aIsKeyDown, aIsKeyRepeat) (((aIsKeyDown) ? 0x2 : 0) | ((aIsKeyRepeat) ? 0x1 : 0))
	
	class IJoystickHatEventHandler
	{
	public:
		virtual void ProcessJoyHatEvent(const SDL_JoyHatEvent& aEvent) = 0;
	};
	
	class IJoystickButtonEventHandler
	{
	public:
		virtual void ProcessJoyButtonEvent(const SDL_JoyButtonEvent& aEvent) = 0;
	};
	
	class EventManager
	{
	public:
		EventManager() { }
		virtual ~EventManager() { }
		
		bool EventLoop(Widget& aWidget);
		void MarkEventLoopDone(bool aLoopSuccess = true) { iLoopSuccess = aLoopSuccess; iDone = true; }
		void ClearEvents();
		
		static void InjectQuitEvent();
		static void InjectRedrawEvent(); // way to inject redraw requests from other threads (e.g. timer callbacks)
		static void InjectCustomCommand(TCommand aCommand);
		static void SendCommand(TCommand aCommand);
		static bool IsLastCommandRepeated();
		
		static void AddKeyTranslation(int aKey, bool aIsKeyDown, bool aIsKeyRepeat, int aCommand) 
		{
			iKeyTranslateTable[std::make_pair(aKey, MAKE_KEY_STATE(aIsKeyDown, aIsKeyRepeat))] = aCommand; 
		}
		
		static void SetJoystickHatEventHandler(IJoystickHatEventHandler* aJoystickHatEventHandler)
		{
			EventManager::iOldJoystickHatEventHandler = EventManager::iCurrentJoystickHatEventHandler;
			EventManager::iCurrentJoystickHatEventHandler = aJoystickHatEventHandler;
		}
		
		static void ResetJoystickHatEventHandler()
		{
			EventManager::iCurrentJoystickHatEventHandler = EventManager::iOldJoystickHatEventHandler;
		}
		
		static void SetJoystickButtonEventHandler(IJoystickButtonEventHandler* aJoystickButtonEventHandler)
		{
			EventManager::iOldJoystickButtonEventHandler = EventManager::iCurrentJoystickButtonEventHandler;
			EventManager::iCurrentJoystickButtonEventHandler = aJoystickButtonEventHandler;
		}
		
		static void ResetJoystickButtonEventHandler()
		{
			EventManager::iCurrentJoystickButtonEventHandler = EventManager::iOldJoystickButtonEventHandler;
		}
		
	protected:
		enum
		{
			ESDLEventRedraw = SDL_USEREVENT, 
			ESDLEventCommand
		};
		
		virtual void ProcessEvent(const SDL_Event& aEvent, Widget& aWidget);
		
		static bool IsSDLEventAlreadyInQueue(Uint32 aEventID);
		
		bool iDone = false;
		bool iLoopSuccess= true;
		
		static Widget* iTopLevelWidget;

		static std::map< std::pair<int, int>, int> iKeyTranslateTable;
		static bool iIsLastCommandRepeated;
		
		static IJoystickHatEventHandler* 	iCurrentJoystickHatEventHandler;
		static IJoystickHatEventHandler* 	iOldJoystickHatEventHandler;
		
		static IJoystickButtonEventHandler* iCurrentJoystickButtonEventHandler;
		static IJoystickButtonEventHandler* iOldJoystickButtonEventHandler;
	};
}

#endif // __SDL_EVENT_MANAGER_H