/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlinfoprint.h"
#include "sdlthemeproperties.h"
#include "sdlsystemrenderer.h"

using namespace sdl;

InfoPrint* InfoPrint::sInfoPrint = nullptr;

static Uint32 infoPrintCallback(Uint32 interval, void *aParam)
{
	InfoPrint* self = (InfoPrint*) aParam;
	self->SetEnabled(false);
	
	EventManager::InjectRedrawEvent();
	
    return(interval);
}

InfoPrint::InfoPrint() : iTimerId(0)
{
	SetName("InfoPrint");
	sInfoPrint = this;
	SystemRenderer::Instance.AddTopLevelWidget(*this);
}

void InfoPrint::ApplyProperties()
{
	iLabel.OverrideDefaults(SDLThemeProperties.iInfoPrint.iMessage);
}

InfoPrint::~InfoPrint()
{
	CloseTimer();
	sInfoPrint = nullptr;
}


void InfoPrint::Construct()
{
	ApplyProperties();
	AddWidget(iLabel);
}

void InfoPrint::CloseTimer()
{
	if (iTimerId != 0)
	{
		SDL_RemoveTimer(iTimerId);
		iTimerId = 0;
	}
}

void InfoPrint::Message(const std::string& aMessage, int aTimeoutInSeconds)
{
	ConstructIfNotAlready();
	
	if (aTimeoutInSeconds == -1)
		aTimeoutInSeconds = SDLThemeProperties.iInfoPrint.iTimeout;
	if (aTimeoutInSeconds <= 0)
		aTimeoutInSeconds = 3;
	
	if (sInfoPrint)
		sInfoPrint->DoMessage(aMessage, aTimeoutInSeconds);
}

void InfoPrint::DoMessage(const std::string& aMessage, int aTimeoutInSeconds)
{
	Open(aMessage);
	iTimerId = SDL_AddTimer(1000 * aTimeoutInSeconds, infoPrintCallback, this);
}

void InfoPrint::Close()
{
	CacheClear();
	CloseTimer();
}

void InfoPrint::Open(const std::string& aMessage)
{
	Close();
	
	int fontSize = iLabel.GetFontSize(); 
	
	int marginW = SDLThemeProperties.iInfoPrint.iRoundedRect.iRadiusX;
	if (marginW < fontSize)
		marginW = fontSize;
	int marginH = SDLThemeProperties.iInfoPrint.iRoundedRect.iRadiusY;
	if (marginH < fontSize)
		marginH = fontSize;

	int screenMargin = 5;
	
	iLabel.SetText(aMessage);
	TSize size = iLabel.GetTextSize();
	
	SetSize(size.iWidth + 2 * marginW, size.iHeight + 2 * marginH);
	iLabel.SetSize(GetSize());
	SetPosition(SDLThemeProperties.iScreen.iWidth - GetSize().iWidth - screenMargin, 100);
	
	SetEnabled(true);
}


void InfoPrint::CreateCachedTextures(IRenderer& aRenderer)
{
	ITexture* texture = RenderUtils::CreateSVGRoundedRectTexture(aRenderer, GetScreenRect().Size(), SDLThemeProperties.iInfoPrint.iRoundedRect);
	if (texture)
		CacheAdd(*texture, TPoint(0, 0));
}

void InfoPrint::ConstructIfNotAlready()
{
	if (!sInfoPrint)
	{
		sInfoPrint = new InfoPrint();
		sInfoPrint->Construct();
	}
}
