/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdllabelwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

void LabelWidget::Properties::OverrideDefaults(const LabelWidget::Properties& aOther)
{
	iFont.OverrideDefaults(aOther.iFont);
	iEclipseText = aOther.iEclipseText;
	OverrideNonEmptyString(iHorizontalAlignment, aOther.iHorizontalAlignment);
	OverrideNonEmptyString(iVerticalAlignment, aOther.iVerticalAlignment);
	iMargin.OverrideDefaults(aOther.iMargin);
}


void LabelWidget::ApplyProperties()
{
	SetHorizontalAlignment(iLabelProperties.iHorizontalAlignment);
	SetVerticalAlignment(iLabelProperties.iVerticalAlignment);
}

void LabelWidget::SetDefaultProperties()
{
	iLabelProperties.iFont = SDLThemeProperties.iDefaultFont; 
	ApplyProperties(); 
}


void LabelWidget::SetFont(const std::string& aFontName, int aFontSize)
{
	OverrideNonEmptyString(iLabelProperties.iFont.iName, aFontName);
	OverridePositiveValue(iLabelProperties.iFont.iSize, aFontSize);
	CacheClear();
}

void LabelWidget::CreateCachedTextures(IRenderer& aRenderer)
{
	if (iText.size() > 0)
	{
		TRect widgetRect(GetSize());
		widgetRect.ApplyMargin(iLabelProperties.iMargin);
		
		ITexture* texture = RenderUtils::CreateTextureFromText(
			aRenderer,
			iLabelProperties.iFont,
			iText, 
			widgetRect.Width(), 
			iLabelProperties.iEclipseText
		);
		
		if (texture)
			CacheAdd(*texture, iLabelProperties.iMargin);
		else
			printf("LabelWidget::CreateCachedTextures, RenderTextToTexture failed, text=%s\n", iText.c_str());
	}
}


TRect LabelWidget::GetRenderedRect() const
{
	return CalcRenderRect(TRect(GetTextSize()), iLabelProperties.iMargin);
}

