/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_LABEL_WIDGET_H
#define __SDL_LABEL_WIDGET_H

#include "sdlwidget.h"

namespace sdl
{
	class LabelWidget : public Widget
	{
	public:
		class Properties
		{
		public:
			Properties(
				const TFont& 		aFont = TFont(),
				bool 				aEclipseText = false, 
				const std::string&	aHorizontalAlignment = "", 
				const std::string&	aVerticalAlignment = "",
				const TMargin 		aMargin = TMargin()
			) 
			: iFont(aFont), iEclipseText(aEclipseText), iHorizontalAlignment(aHorizontalAlignment), iVerticalAlignment(aVerticalAlignment), iMargin(aMargin) {}
			void OverrideDefaults(const Properties& aOther);
			TFont 		iFont;
			bool 		iEclipseText;
			std::string	iHorizontalAlignment;
			std::string	iVerticalAlignment;
			TMargin 	iMargin;
		};
		
		LabelWidget()  { SetDefaultProperties(); }
		~LabelWidget() {}
		
		void SetFont(const std::string& aFontName, int aFontSize);
		void SetText(const std::string& aText) 			{ iText = aText; CacheClear(); }
		void SetTextColor(const TColor& aColor)			{ iLabelProperties.iFont.iTextColor.OverrideDefaults(aColor); CacheClear(); }
		void SetBold(bool aEnable=true) 				{ iLabelProperties.iFont.iBold = aEnable; CacheClear(); }
		void SetItalic(bool aEnable=true) 				{ iLabelProperties.iFont.iItalic = aEnable; CacheClear(); }
		void SetUnderline(bool aEnable=true) 			{ iLabelProperties.iFont.iUnderline = aEnable; CacheClear(); }
		void SetStrikeThrough(bool aEnable=true) 		{ iLabelProperties.iFont.iStrikeThrough= aEnable; CacheClear(); }
		void SetEclipseText(bool aEclipseText)			{ iLabelProperties.iEclipseText = aEclipseText; CacheClear(); }
		void SetMargin(const TMargin& aMargin)			{ iLabelProperties.iMargin.OverrideDefaults(aMargin); CacheClear(); }
		void SetMargin(int aMargin)						{ SetMargin(TMargin(aMargin)); }
		void SetOutline(int aOutline)					{ OverridePositiveValue(iLabelProperties.iFont.iOutline, aOutline); CacheClear(); }

		const TFont& GetFont() const 					{ return iLabelProperties.iFont; }
		int   GetFontSize() const 						{ return iLabelProperties.iFont.iSize; }
		const std::string&	GetText() const				{ return iText; }
		TMargin GetMargin() const 						{ return iLabelProperties.iMargin; }
		TSize GetTextSize() const						{ return RenderUtils::GetTextSize(iLabelProperties.iFont, iText); } 
		virtual int GetMinimumHeight() const override	{ return (int) (iLabelProperties.iFont.iSize * 1.5); }
		TRect GetRenderedRect() const;
		void OverrideDefaults(const Properties& aOther) { iLabelProperties.OverrideDefaults(aOther); ApplyProperties(); }
		
	protected:
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;
		
		void SetDefaultProperties();
		void ApplyProperties();
		
		std::string		iText;
		Properties		iLabelProperties;
	};
}

#endif // __SDL_LABEL_WIDGET_H