/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdllayout.h"
#include "sdlcontainer.h"

using namespace sdl;


void LayoutManager::ListLayout(Container& aContainer)
{
	int width = aContainer.GetClientAreaSize().iWidth;
	TPoint p(0, 0);
	for (auto &widget : aContainer.iWidgets)
	{
		int height = widget->GetMinimumHeight();
		widget->SetRect(TRect(p, TSize(width, height)));
		widget->Layout();
		p.iY += height;
	}
}



void LayoutManager::GridLayout(Container& aContainer, int aColumns, bool aResizeHeight)
{
	TSize size = aContainer.GetClientAreaSize();
	
	float width =  ((float) size.iWidth) / aColumns;
	
	int col = 0;
	int height = 0;
	int totalHeight = 0;
	TPoint point(0, 0);
	
	for (auto &widget : aContainer.iWidgets)
	{
		widget->SetPosition(point);
		int mh = widget->GetMinimumHeight();
		widget->SetSize((int) width, mh);
		if (mh > height)
			height = mh;
		
		point.iX += (int) width;
		
		if (++col % aColumns == 0)
		{
			point.iX = 0;
			point.iY += height;
			totalHeight += height;
			height = 0;
		}
	}
	
	if (aResizeHeight)
	{
		totalHeight += height; // add height of current row, if new row then height is 0
		
		TSize size = aContainer.GetSize();
		int delta = size.iHeight - totalHeight;
		size.iHeight -= delta;
		aContainer.SetSize(size);
	}
}


void LayoutManager::RowLayout(Container& aContainer, THorAlignment aHorAlignment, TVertAlignment aVertAlignment)
{
	const int spacing = 15;
	TSize size = aContainer.GetClientAreaSize();
	
	int width = 0;
	for (auto &widget : aContainer.iWidgets)
		width += widget->GetSize().iWidth;
	
	width += (aContainer.iWidgets.size()-1) * spacing;
	int left = 0;
	switch (aHorAlignment)
	{
		case THorAlignment::ECenter:
			left += (size.iWidth - width) /2;
			break;
		case THorAlignment::ERight:
			left += size.iWidth - width;
			break;
		case THorAlignment::ELeft:
		default:
			// nothing to do
			break;
	}
	
	TPoint point(left, 0);
	int height = 0;
	for (auto &widget : aContainer.iWidgets)
	{
		TSize size = widget->GetSize(); 
		size.iHeight = widget->GetMinimumHeight();
		
		switch (aVertAlignment)
		{
			case TVertAlignment::ECenter:
				point.iY = (size.iHeight - size.iHeight) / 2;
				break;
			case TVertAlignment::ETop:
				point.iY = 0;
				break;
			case TVertAlignment::EBottom:
				point.iY = size.iHeight - size.iHeight;
				break;
			default:
				// nothing to do
				break;
		}
		widget->SetPosition(point);
		
		if (size.iHeight > height)
			height = size.iHeight;
		widget->SetSize(size);
		
		point.iX += spacing + size.iWidth;
	}
}
