/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlpopupmenuwidget.h"
#include "sdleventmanager.h"
#include "sdllabelwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;



PopupMenuWidget::PopupMenuWidget() : iMenu("popup")
{
	SetDefaultProperties();
}

void PopupMenuWidget::SetDefaultProperties()
{
	iPopupMenuProperties = SDLThemeProperties.iDefaultPopupMenuProperties;
	ApplyProperties();
}


TRect PopupMenuWidget::GetClientArea() const
{
	TRect clientAreaRect;
	clientAreaRect.SetSize(GetSize() - TSize(iPopupMenuProperties.iRoundedRect.iRadiusX*2, iPopupMenuProperties.iRoundedRect.iRadiusY *2));
	clientAreaRect.SetPosition(TPoint(iPopupMenuProperties.iRoundedRect.iRadiusX, iPopupMenuProperties.iRoundedRect.iRadiusX));
	clientAreaRect.ApplyMargin(iPopupMenuProperties.iRoundedRect.iRadiusX);
	
	return clientAreaRect; 
}

void PopupMenuWidget::SizeAndPosition(const TSize& aSize)
{
	int		rightX = GetPosition().iX + GetSize().iWidth;
	TSize	clientArea = GetClientAreaSize();
	
	clientArea.iWidth = aSize.iWidth * 1.5;
	
	if (aSize.iHeight <= clientArea.iHeight)
		clientArea.iHeight = aSize.iHeight;
	else
	{
		int h = aSize.iHeight / iMenu.GetMenuItems().size();
		clientArea.iHeight /= h;
		clientArea.iHeight *= h;
	}
	
	ResizeToClientArea(clientArea);
	
	Layout();
	
	SetPosition(TPoint(rightX - GetSize().iWidth, GetPosition().iY));
}


void PopupMenuWidget::Setup()
{
	AddWidget(iList);
	for (auto &item : iMenu.GetMenuItems())
		iList.AddItem(item.get()->GetName());
	
	Layout();
	
	SizeAndPosition(iList.CalculateMaxSize());
}

bool PopupMenuWidget::Execute()
{
	Setup();
	
	SwitchFocus(FindFocusNext());
	
	bool success = iEventManager.EventLoop(*this);
	
	return success;
}

bool PopupMenuWidget::HandleCommandMyself(TCommand aCommand)
{
	bool consumed = false;
	
	switch (aCommand)
	{
		case TCommand::EBack:
		case TCommand::ECancel:
		{
			iEventManager.MarkEventLoopDone(false);
			break;
		}
		
		case TCommand::ESelect:
		{
			iEventManager.MarkEventLoopDone(true);
			break;
		}
		
		default:
		{
			consumed = Container::HandleCommandMyself(aCommand);
			break;
		}
	}
	return consumed;
}

void PopupMenuWidget::CreateCachedTextures(IRenderer& aRenderer)
{
	ITexture* texture = RenderUtils::CreateSVGRoundedRectTexture(aRenderer, GetScreenRect().Size(), iPopupMenuProperties.iRoundedRect);
	if (texture)
		CacheAdd(*texture, TPoint(0,0));
}

