/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_POPUP_MENU_WIDGET_H
#define __SDL_POPUP_MENU_WIDGET_H

#include "sdlcore.h"
#include "sdleventmanager.h"
#include "sdlmenu.h"
#include "sdlcontainer.h"
#include "sdllistwidget.h"

namespace sdl 
{
	class PopupMenuWidget : public Container
	{
	public:
		class Properties
		{
		public:
			Properties(const LabelWidget::Properties& aLabel = LabelWidget::Properties(), const TRoundedRect& aRoundedRect = TRoundedRect())
			: iTextLabels(aLabel), iRoundedRect(aRoundedRect) { }
			void OverrideDefaults(const Properties& aOther)
			{
				iTextLabels.OverrideDefaults(aOther.iTextLabels);
				iRoundedRect.OverrideDefaults(aOther.iRoundedRect);
			}
			LabelWidget::Properties	iTextLabels;
			TRoundedRect			iRoundedRect;
		};
		
		PopupMenuWidget();
		
		Menu& GetMenu()									{ return iMenu; }
		int GetHighlightedItemIndex() 					{ return iList.GetFocussedItemIndex() ; }
		
		void OverrideDefaults(const Properties& aOther) { iPopupMenuProperties.OverrideDefaults(aOther); ApplyProperties(); }
		
		bool Execute();
		
	protected:
		void SetDefaultProperties();
		void ApplyProperties() { }
		
		virtual bool HasInput() const override { return true; } // otherwise no up/down focus
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;
		virtual bool HandleCommandMyself(TCommand aCommand) override;
		void Setup();
		
		void SizeAndPosition(const TSize& aSize);
		virtual TRect GetClientArea() const override;
		
		Properties 		iPopupMenuProperties;
		Menu			iMenu;
		EventManager	iEventManager;
		ListWidget		iList;
	};
}

#endif // __SDL_POPUP_MENU_WIDGET_H