/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SCROLLABLE_CONTAINER_H
#define __SDL_SCROLLABLE_CONTAINER_H

#include "sdlcore.h"
#include "sdlcontainer.h"
#include "sdlscrollbarwidget.h"

namespace sdl
{
	class ScrollableContainer : public Container
	{
	public:
		enum class TLineItemModel
		{
			EFixedHeight,
			EVariableHeight
		};
		
		ScrollableContainer() {  }
		void SetLineItemModel(TLineItemModel aLineItemModel) { iLineItemModel = aLineItemModel; }
		virtual void Layout() override;
		void Reset() { iScrollbar.Reset(); }
	protected:
		virtual bool HandleCommandMyself(TCommand aCommand);
		
		virtual void Render(IRenderer& aRenderer) override;
		
		bool IsAtTop() const				{ return iViewPortStartLine == 0; }
		bool IsAtBottom() const				{ return iViewPortStartLine + iViewPortNumberOfLines == GetNumberOfLines(); }
		int GetLineHeight(int aLine) const  { return iWidgets[aLine]->GetSize().iHeight; }
		int GetNumberOfLines() const		{ return iWidgets.size(); }
		int GetLineHeight(int aStartLine, int aNumOfLines) const;
		
		bool ViewPortScrollUp();
		bool ViewPortScrollDown();
		void ViewPortScrollToTop();
		void ViewPortScrollToBottom();
		
		virtual void PreLayout() { }
		virtual void PostLayout();
		void ReLayoutIfNeeded();
		void SetContainerPosition();
		
		virtual void SizeChanged(const TSize& aOldSize) override;
		virtual void PositionChanged(const TPoint& aOldPosition) override { PositionScrollbar(); Container::PositionChanged(aOldPosition); }
	
		virtual void HightlightChanged(THighlightChange aHighlightChange) override;
		
		int GetViewPortStartLine() const 		{ return iViewPortStartLine; }
		int GetViewPortNumberOfLines() const	{ return iViewPortNumberOfLines; }
		
		virtual TRect GetClientArea() const;
		
		ScrollbarWidget iScrollbar;
	private:
		void PositionScrollbar();
		void UpdateScrollbar();
		bool NeedsScrollbar();
		
		bool ViewPortAddLine(int aLine, int& aCurrentheight);
		void SetTopLine(int aLine);

		void SetBottomLine(int aLine);
		int GetUnusedViewPortHeight();

		void ResizeForFixedHeightItems();

		TLineItemModel	iLineItemModel = TLineItemModel::EFixedHeight;
		int				iViewPortStartLine = 0;
		int				iViewPortNumberOfLines = 0;
		TPoint			iOriginalPosition;
	};
}

#endif // __SDL_SCROLLABLE_CONTAINER_H