/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlscrollbarwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

void ScrollbarWidget::SetDefaultProperties()
{
	iScrollBarPoperties.iRoundedRect.iRadiusX = 5;
	iScrollBarPoperties.iRoundedRect.iRadiusY = 5;
	iScrollBarPoperties.iRoundedRect.iFrameColor = SDLThemeProperties.iControlProperties.iFrameColor;
	iScrollBarPoperties.iRoundedRect.iFillColor = SDLThemeProperties.iControlProperties.iScrollbarThumb;
	iScrollBarPoperties.iRoundedRect.iFrameWidth = 2;

	iScrollBarPoperties.iWidthSmallScrollbar = 3;
	iScrollBarPoperties.iWidthNormalScrollbar = SDLThemeProperties.iDefaultFont.iSize/2;
	
	SetFormat(TFormat::ENormal) ;
}

void ScrollbarWidget::SetFormat(const TFormat aFormat) 
{
	iFormat = aFormat; 
	if (iFormat == TFormat::ENormal)
		SetWidth(iScrollBarPoperties.iWidthNormalScrollbar);
	else
		SetWidth(iScrollBarPoperties.iWidthSmallScrollbar);
}



void ScrollbarWidget::RenderSmall(IRenderer& aRenderer)
{
	TRect rect = iModel.GetThumbRect(GetSize());
	rect.SetPosition(GetScreenPosition(rect.Position())); // convert to screen coordinates
	
	aRenderer.RenderFillRect(rect, iScrollBarPoperties.iThumbColor);
}

void ScrollbarWidget::RenderNormal(IRenderer& aRenderer)
{
	int stroke = 2;
	TRect rect = GetScreenRect();
	rect.SetHeight(stroke);
	aRenderer.RenderFillRect(rect, iScrollBarPoperties.iFrameColor);
	
	rect = GetScreenRect();
	rect += TPoint(0, rect.Height() - stroke);
	rect.SetHeight(stroke);
	aRenderer.RenderFillRect(rect, iScrollBarPoperties.iFrameColor);
	
	rect = GetScreenRect();
}


void ScrollbarWidget::Render(IRenderer& aRenderer)
{
	if (IsEnabled())
	{
		Widget::Render(aRenderer);
		if (iFormat == TFormat::ENormal)
			RenderNormal(aRenderer);
		else
			RenderSmall(aRenderer);
	}
}


TRect ScrollbarWidget::Model::GetThumbRect(const TSize& aScrollbarSize) const
{
	TRect thumbRect = TRect(aScrollbarSize);
	
	//int top = GetModelHeight() - iModelTop;
	int top = iModelTop;
	float ratio = (float) aScrollbarSize.iHeight / (float) GetModelHeight();
	int y = (top * thumbRect.Height()) / GetModelHeight();
	
	thumbRect += TPoint(0, y);
	thumbRect.SetHeight(thumbRect.Height() * ratio);
	
	return thumbRect;
}

void ScrollbarWidget::ThumbUpdated()
{
	if (IsEnabled())
	{
		TRect rect = iModel.GetThumbRect(GetSize());
		CacheSetPosition(0, rect.Position());
	}
}

void ScrollbarWidget::CreateCachedTextures(IRenderer& aRenderer)
{
	TRect rect = iModel.GetThumbRect(GetSize());
	
	ITexture* texture = RenderUtils::CreateSVGRoundedRectTexture(aRenderer, rect.Size(), iScrollBarPoperties.iRoundedRect);
	if (texture)
		CacheAdd(*texture, rect.Position());
}
