/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SCROLLBAR_WIDGET_H
#define __SDL_SCROLLBAR_WIDGET_H

#include "sdlcore.h"
#include "sdlwidget.h"

namespace sdl 
{
	class ScrollbarWidget : public Widget
	{
	public:
		enum class TFormat
		{
			ENormal,
			ESmall
		};

		class Properties
		{
		public:
			TColor 	iFrameColor;
			TColor  iThumbColor;
			void OverrideDefaults(const Properties& aOther)
			{
				iRoundedRect.OverrideDefaults(aOther.iRoundedRect);
			}
			TRoundedRect iRoundedRect;
			
			int iWidthSmallScrollbar;
			int iWidthNormalScrollbar;
		};
		ScrollbarWidget() { SetDefaultProperties(); } 
		
		void Reset()						{ iIsEnabled = false;  }
		void SetModelHeight(int aHeight)	{ iModel.SetModelHeight(aHeight); CacheClear(); }
		void SetModelTop(int aY)			{ iModel.SetModelTop(aY); ThumbUpdated(); }
		
		bool IsEnabled() const 				{ return iIsEnabled; }
		void Enable(bool aEnable=true) 		{ iIsEnabled = aEnable; }
		void SetFormat(const TFormat aFormat);
		
		virtual void Render(IRenderer& aRenderer) override;
	protected:
		void RenderNormal(IRenderer& aRenderer);
		void RenderSmall(IRenderer& aRenderer);
		
		void SetDefaultProperties();
		void ApplyProperties() { }
		
		void ThumbUpdated();
		
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;
		
		class Model
		{
		public:
			TRect GetThumbRect(const TSize& aScrollbarSize) const;

			void SetModelHeight(int aHeight)			{ iModelHeight = aHeight; }
			void SetModelTop(int aY)					{ iModelTop = aY; }
			void Reset()								{ iModelHeight = iModelTop =0; }
		private:
			int 	GetModelHeight() const				{ return (iModelHeight==0) ? 1 : iModelHeight; }
			int 	iModelHeight = 0;
			int 	iModelTop = 0;
		};
		Model		iModel;
		bool		iIsEnabled = false;
		TFormat		iFormat = TFormat::ENormal;
		Properties	iScrollBarPoperties;
	};
}

#endif // __SDL_SCROLLBAR_WIDGET_H