/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SIMPLE_TEXT_WIDGET_H
#define __SDL_SIMPLE_TEXT_WIDGET_H

#include "sdlscrollablecontainer.h"
#include "simpletextview.h"

namespace sdl 
{
class SimpleTextWidget : public ScrollableContainer
{
	public:
		class Properties
		{
		public:
			Properties()  { }
			void OverrideDefaults(const Properties& aOther)
			{
				OverrideNonEmptyString(iVerticalAlign, aOther.iVerticalAlign);
				iSingleLine = aOther.iSingleLine;
				iEclipseText = aOther.iEclipseText;
			}
			std::string iVerticalAlign;
			bool		iSingleLine;
			bool		iEclipseText;
		};
		
		SimpleTextWidget() { SetDefaultProperties(); }
		void SetText(const std::string& aText)		{ iSimpleTextModel.SetText(aText); Reset(); }
		void LimitToOneLine(bool aEclipseLastWord)	{ iSimpleTextModel.LimitToOneLine(aEclipseLastWord);  Reset(); }

		int GetTotalHeight() const;
		void OverrideDefaults(const Properties& aOther) { iSimpleTextProperties.OverrideDefaults(aOther); ApplyProperties(); } 
	protected:
		void ApplyProperties();
		void SetDefaultProperties();
		
		void AddLineWidgets();
		virtual void PreLayout() override;
		virtual TRect GetClientArea() const;
		
		class SimpleTextLineWidget : public Widget, public simpletext::IViewRenderer
		{
		public:
			SimpleTextLineWidget(int aLine, simpletext::View& aSimpleTextView);
			virtual void Render(IRenderer& aRenderer) override;
			virtual int GetMinimumHeight() const override;
		protected: // implements simpletext::IViewRenderer
			virtual int RenderWord(int aX, int aY, const std::string& aWord, const simpletext::TCharFormat& aCharFormat) override;
			
		private:
			int					iLine;
			simpletext::View&	iSimpleTextView;
			IRenderer*			iRenderer = nullptr;
		};
		
		Properties iSimpleTextProperties;
		
	private: 
		simpletext::Model	iSimpleTextModel;
		simpletext::View 	iSimpleTextUI = simpletext::View(iSimpleTextModel);
	};
}

#endif // __SDL_SIMPLE_TEXT_WIDGET_H