/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SPINNER_WIDGET_H
#define __SDL_SPINNER_WIDGET_H

#include "sdlcore.h"
#include "sdllabelwidget.h"

namespace sdl
{
	class SpinnerWidget : public Widget
	{
	public:
		class SpinnerModel
		{
		public:
			virtual std::string ToString() const = 0;
			virtual bool Next() = 0;
			virtual bool Previous() = 0;
			virtual bool GetValueList(std::vector<std::string>& aValueList) = 0;
			virtual void SetValueIndex(int aIndex) = 0;
			virtual std::string GetMaxWidthSampleText() = 0;
		};
		
		class SpinnerModelInt : public SpinnerModel
		{
		public:
			SpinnerModelInt(int aValue=0, int aIncrement=1, int aMinValue=-1000, int aMaxValue=1000) : 
				iValue(aValue), iIncrement(aIncrement), iMinValue(aMinValue), iMaxValue(aMaxValue) { }
				
			virtual std::string ToString() const override	{ return std::to_string(iValue); }
			virtual bool Next() override					{ return SetValue(iValue + iIncrement); }
			virtual bool Previous() override				{ return SetValue(iValue - iIncrement); }
			virtual void SetValueIndex(int aIndex) override { }
			virtual std::string GetMaxWidthSampleText() {return "99999"; }
			
			virtual bool GetValueList(std::vector<std::string>& aValueList) override { return false; }
			
			void SetRange(int aMinValue, int aMaxValue)		{ iMinValue = aMinValue; iMaxValue = aMaxValue; }
			void SetIncrement(int aIncrement)				{ iIncrement = aIncrement; }
			bool SetValue(int aValue)						{ int old = iValue; iValue = ClipValue(aValue); return old!=iValue; }
			int GetValue() const 							{ return iValue; }
			
		protected:
			int ClipValue(int aValue);
			int iValue;
			int iIncrement;
			int iMinValue;
			int iMaxValue;
		};
		
		class SpinnerModelList : public SpinnerModel
		{
		public:
			SpinnerModelList() { }
			virtual std::string ToString() const override;
			virtual bool Next() override;
			virtual bool Previous() override;
			virtual bool GetValueList(std::vector<std::string>& aValueList) override;
			virtual void SetValueIndex(int aIndex) { iIndex=aIndex; }
			virtual std::string GetMaxWidthSampleText() {return "Sample text"; }
			
			void SetCurrentString(const std::string& aString);
			void AddString(const std::string& aString) { iList.push_back(aString); }
			void SetList(std::vector<std::string>& aList) { iList.clear(); iList = aList; }
		protected:
			std::vector<std::string> iList;
			int iIndex = 0;
		};
		
		SpinnerWidget(SpinnerModel& aSpinnerModel);
		~SpinnerWidget() {}
		
		void SetLabelText(const std::string& aText) { iLabel.SetText(aText); }
		void SetMaxWidthValueString(const std::string& aSampleText);
	protected:
		void Update();
		void Popup();
		
		virtual void Layout() override			{ Update(); }
		virtual bool HasInput() const override	{ return true; }
		
		virtual void SizeChanged(const TSize& aOldSize) override;
		virtual void Render(IRenderer& aRenderer) override;
		virtual bool HandleCommand(TCommand aCommand) override;
		virtual int GetMinimumHeight() const override { return (int) (iLabel.GetFontSize() * 1.5); }
		void CreateArrowTextureIfNeeded(IRenderer& aRenderer);
		
		TSize GetArrowSpaceSize() const { int height = GetMinimumHeight(); return TSize(height * sArrowRatio, height); }
		
		SpinnerModel&	iSpinnerModel;
		LabelWidget		iLabel;
		LabelWidget		iLabelValue;
		int				iMaxSpinnerWidth;
		static float 	sArrowRatio;
		
		std::unique_ptr<ITexture> iArrowTexture = nullptr;
	};
}

#endif // __SDL_SPINNER_WIDGET_H