/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlswitchwidget.h"
#include "sdlthemeproperties.h"

using namespace sdl;

SwitchWidget::SwitchWidget()
{
	iLabel.SetAlignment(LabelWidget::THorizontalAlignment::ELeft);
	iLabel.SetAlignment(LabelWidget::TVerticalAlignment::ECenter);
	iLabel.SetMargin(TMargin((int) (SDLThemeProperties.iDefaultFont.iSize) / 2, 0, 0, 0));
	
	iLabelValue.SetAlignment(LabelWidget::THorizontalAlignment::ERight);
	iLabelValue.SetAlignment(LabelWidget::TVerticalAlignment::ECenter);
	iLabelValue.SetMargin(TMargin(0, 0, (int) (SDLThemeProperties.iDefaultFont.iSize / 2) + GetSwitchSpace().iWidth , 0));
}

void SwitchWidget::Render(IRenderer& aRenderer)
{
	Widget::Render(aRenderer);
	
	const TRect screenRect = GetScreenRect();
	
	iLabel.SetTextColor(HasFocus() ? SDLThemeProperties.iDefaultHighlightTextColor : SDLThemeProperties.iDefaultFont.iTextColor);
	iLabelValue.SetTextColor(HasFocus() ? SDLThemeProperties.iDefaultHighlightTextColor : SDLThemeProperties.iDefaultFont.iTextColor);
	
	iLabel.SetPosition(screenRect.Position());
	iLabel.Render(aRenderer);
	
	iLabelValue.SetPosition(screenRect.Position());
	iLabelValue.Render(aRenderer);
}

bool SwitchWidget::HandleCommand(TCommand aCommand)
{
	bool consumed = false;
	
	if (HasFocus())
	{
		switch(aCommand)
		{
			case TCommand::ESelect:
			{
				SetEnabled(!iEnabled);
				consumed = true;
				break;
			}
			case TCommand::ELeft:
			{
				SetEnabled(false);
				consumed = true;
				break;
			}
			case TCommand::ERight:
			{
				SetEnabled(true);
				consumed = true;
				break;
			}
			default:
				break;
		}
	}
	if (consumed)
		System::RequestReRender();
		
	return consumed;
}

void SwitchWidget::CreateCachedTextures(IRenderer& aRenderer)
{
	TRect frameRect;
	CreateCachedTextureFrame(aRenderer, frameRect);
	CreateCachedTextureButton(aRenderer, frameRect);
}

void SwitchWidget::CreateCachedTextureFrame(IRenderer& aRenderer, TRect& aRect)
{
	TColor frameColor = SDLThemeProperties.iControlProperties.iFrameColor;
	TColor fillColor = IsEnabled() ?  SDLThemeProperties.iControlProperties.iFillColorOn : SDLThemeProperties.iControlProperties.iFillColorOff; 
	
	TRoundedRect roundedRect(10, 10, frameColor, fillColor, 2);
	TSize size(GetSwitchSpace());
	size.iHeight *= .65f;
	ITexture* texture = RenderUtils::CreateSVGRoundedRectTexture(aRenderer, size, roundedRect);
	
	if (texture)
	{
		// draw centered in the right-margin space of the value label
		int rightMargin = iLabelValue.GetMargin().iRight;
		const TRect widgetRect = GetSize();
		const TRect space(widgetRect.Position() + TPoint(widgetRect.Width() - rightMargin, 0), TSize(rightMargin, widgetRect.Height()));
		aRect = texture->GetRect();
		aRect.Center(space);
		CacheAdd(*texture, aRect.Position());
	}
}

void SwitchWidget::CreateCachedTextureButton(IRenderer& aRenderer, const TRect& aRect)
{
	TColor frameColor = SDLThemeProperties.iControlProperties.iFrameColor;
	TColor fillColor = 	IsEnabled() ? SDLThemeProperties.iControlProperties.iFrameColor : SDLThemeProperties.iControlProperties.iFillColorOff; 
	
	int margin = 6;
	TSize size(TSize(aRect.Height() - margin));
	
	ITexture* texture = RenderUtils::CreateSVGCircleTexture(aRenderer, size, 2, frameColor, fillColor);
	
	if (texture)
	{
		TRect rect = texture->GetRect();
		if (IsEnabled())
			rect.SetPosition(aRect.Position() + TPoint(aRect.Width() - rect.Width(), 0) + TPoint(-margin/2, margin/2));
		else
			rect.SetPosition(aRect.Position() + TPoint(margin/2));
		
		CacheAdd(*texture, rect.Position());
	}
}

