/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_SWITCH_WIDGET_H
#define __SDL_SWITCH_WIDGET_H

#include "sdlcore.h"
#include "sdllabelwidget.h"

namespace sdl
{
	class SwitchWidget : public Widget
	{
	public:
		SwitchWidget();
		
		void SetLabelText(const std::string& aText)		{ iLabel.SetText(aText); }
		const std::string& GetText() const				{ return iLabel.GetText(); }
		void SetEnabled(bool aEnabled)					{ iEnabled = aEnabled; StateUpdated(); }
		bool IsEnabled() const							{ return iEnabled; }
		void SetOnText(const std::string& aOnText)		{ iOnText = aOnText; }
		void SetOffText(const std::string& aOffText)	{ iOffText = aOffText; }
		
		virtual bool HasInput() const override			{ return true; }
		virtual int GetMinimumHeight() const override	{ return (int) (iLabel.GetFontSize() * 1.5); }
		
	protected:
		virtual bool HandleCommand(TCommand aCommand) override;
		virtual void Render(IRenderer& aRenderer) override;
		
		virtual void SizeChanged(const TSize& aOldSize) override	{ iLabel.SetSize(GetSize()); iLabelValue.SetSize(GetSize()); Widget::SizeChanged(aOldSize); }
		void StateUpdated()											{ iLabelValue.SetText(iEnabled ? iOnText : iOffText);  CacheClear(); }
		TSize GetSwitchSpace() const								{ return TSize(GetMinimumHeight() * 1.5, GetMinimumHeight()); }
		
		LabelWidget	iLabel;
		LabelWidget	iLabelValue;
		bool 		iEnabled = false;
		std::string iOnText = "ON";
		std::string iOffText = "OFF";
		
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;
		void CreateCachedTextureFrame(IRenderer& aRenderer, TRect& aRect);
		void CreateCachedTextureButton(IRenderer& aRenderer, const TRect& aRect);
	};
}

#endif //  __SDL_SWITCH_WIDGET_H