/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlsystemrenderer.h"
#include "renderers/renderfactory.h"
#include "sdlthemeproperties.h"
#include "sdlsystemproperties.h"

using namespace sdl;

SystemRenderer SystemRenderer::Instance;

static IRenderer* iRenderer = nullptr;

void SystemRenderer::Create(const std::string& aWindowTitle)
{
	TSize size(SDLThemeProperties.iScreen.iWidth, SDLThemeProperties.iScreen.iHeight);
	iRenderer = RenderFactory::CreateRenderer(SDLSystemProperties.iRenderer, size, aWindowTitle, SDLThemeProperties.iScreen.iFullScreen);
}

void SystemRenderer::Destroy()
{
	if (iRenderer)
	{
		delete iRenderer;
		iRenderer = nullptr;
	}
}

IRenderer* SystemRenderer::GetRenderer()
{
	return iRenderer;
}


void SystemRenderer::Render()
{
	iRenderer->Clear();
	RenderWidgets(*iRenderer);
	iRenderer->Present();
}

void SystemRenderer::RenderWidgets(IRenderer& aRenderer)
{
	for (auto &widget : iWidgets)
		widget->Render(aRenderer);
	for (auto &widget : iTopLevelWidgets)
		widget->Render(aRenderer);
}



