/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_THEME_PROPERTIES_H
#define __SDL_THEME_PROPERTIES_H

#include "sdlcore.h"
#include "sdllabelwidget.h"
#include "sdlwindow.h"
#include "sdldialog.h"
#include "sdlinfoprint.h"
#include "sdlpopupmenuwidget.h"
#include "sdlmenu2dwidget.h"
#include "sdlaudio.h"
#include "platform.h"

namespace sdl
{

class ThemeProperties
{
public:
	ThemeProperties() { SetDefaults(); }
	
	class TScreen
	{
	public:
		int		iWidth;
		int		iHeight;
		bool	iFullScreen;
	};
	
	class TDebug
	{
	public:
		bool	iShowOutline;
		TColor	iOutlineColor;
	};
	
	class TControl
	{
	public:
		TColor 	iFrameColor;
		int 	iFrameSize;
		TColor 	iFillColorOn;
		TColor 	iFillColorOff;
		TColor 	iScrollbarThumb;
	};

	class InfoPrintProperties 
	{
	public:
		TRoundedRect			iRoundedRect;
		LabelWidget::Properties iMessage;
		int						iTimeout = 3;
	};
	
	TScreen 					iScreen;
	
	// non-highlight background & text
	TFont 						iDefaultFont;
	TColor						iDefaultBackgroundColor;
	
	// highlight background & text
	TColor						iDefaultHighlightBackgroundColor;
	TColor						iDefaultHighlightTextColor;
	
	Window::Properties			iDefaultWindowProperties;
	PopupMenuWidget::Properties	iDefaultPopupMenuProperties;
	Menu2DWidget::Properties	iDefaultMenu2DUIProperties;
	Dialog::Properties			iDefaultDialogProperties;
	InfoPrintProperties 		iInfoPrint;
	TControl					iControlProperties;
	TDebug						iDebug;
	
	void SetDefaults()
	{
		iScreen 							= {800, 600, false};
		iDefaultFont 						= {"Arimo-Regular.ttf", 24, TColor::KWhite};
		iDefaultBackgroundColor 			= 0x969696;
		iDefaultHighlightBackgroundColor	= 0xc0c0c0;
		iDefaultHighlightTextColor 			= 0;
		
		iDefaultWindowProperties 	= {{{"", -1}, false, "center", "center"}, -1, {25, 25, 0, TColor::KUndefinedColor, 3}};
		iDefaultPopupMenuProperties = {{{"", -1}, false, "left", "center"}, {5, 5, 0, TColor::KUndefinedColor, 3}};

		iDefaultMenu2DUIProperties 	= Menu2DWidget::Properties(); 
		iDefaultDialogProperties 	= Dialog::Properties();
		
		iInfoPrint.iRoundedRect = TRoundedRect();
		iInfoPrint.iMessage = LabelWidget::Properties();
		iInfoPrint.iMessage.iHorizontalAlignment = iInfoPrint.iMessage.iVerticalAlignment = "center";
		iInfoPrint.iTimeout =  3;
		
		iControlProperties			= {TColor::KUndefinedColor, 3, 0xffff, 0xffffff, TColor::KUndefinedColor};
		iDebug 						= {false, 0xff0000};
	}
};

//class Properties;
extern ThemeProperties SDLThemeProperties;


} // namespace sdl

#endif // __SDL_THEME_PROPERTIES_H