/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#include "sdlwindow.h"
#include "sdlthemeproperties.h"

using namespace sdl;

void Window::SetDefaultProperties()
{
	iWindowProperties = SDLThemeProperties.iDefaultWindowProperties;
	iWindowProperties.iLabelTitle.iFont.SetDefaults(SDLThemeProperties.iDefaultFont);
	iWindowProperties.iTitleHeight = iWindowProperties.iLabelTitle.iFont.iSize * 2;
	ApplyProperties();
}


TRect Window::GetClientArea() const
{
	TRect clientAreaRect;
	clientAreaRect.SetSize(GetSize() - TSize(iWindowProperties.iRoundedRect.iRadiusX*2, iWindowProperties.iRoundedRect.iRadiusY *2) - TSize(0, iLabelTitle.GetSize().iHeight));
	clientAreaRect.SetPosition(TPoint((iWindowProperties.iRoundedRect.iRadiusX), (iWindowProperties.iRoundedRect.iRadiusX) + iLabelTitle.GetSize().iHeight));
	clientAreaRect.ApplyMargin(iInternalMargin);
	
	return clientAreaRect; 
}

void Window::SizeChanged(const TSize& aOldSize)
{
	iLabelTitle.SetSize(TSize(GetSize().iWidth - (2 * iWindowProperties.iRoundedRect.iRadiusX), iWindowProperties.iTitleHeight));
	
	if (iAutoCenter)
		CenterWindow();
	Container::SizeChanged(aOldSize);
}

void Window::CenterWindow()
{
	TRect screenRect(TSize(SDLThemeProperties.iScreen.iWidth, SDLThemeProperties.iScreen.iHeight));
	TRect windowRect = GetRect();
	windowRect.Center(screenRect);
	SetPosition(windowRect.Position());
}


void Window::PositionChanged(const TPoint& aOldPosition)
{
	iLabelTitle.SetPosition(TPoint(iWindowProperties.iRoundedRect.iRadiusX, iWindowProperties.iRoundedRect.iFrameWidth) + GetScreenPosition()); 
	Container::PositionChanged(aOldPosition);
}

void Window::CreateCachedTextures(IRenderer& aRenderer)
{
	ITexture* texture = RenderUtils::CreateSVGRoundedRectTexture(aRenderer, GetScreenRect().Size(), iWindowProperties.iRoundedRect);
	if (texture)
		CacheAdd(*texture, TPoint(0, 0));
}

void Window::Render(IRenderer& aRenderer)
{
	Container::Render(aRenderer);
	
	// draw title
	iLabelTitle.Render(aRenderer);
	
	// draw divider bar in same color and thickness as the window frame
	TRect screenRect = GetScreenRect();
	TRect rect =  iLabelTitle.GetRect();
	
	rect.SetPosition(TPoint(screenRect.X(), rect.Y() + rect.Height() - iWindowProperties.iRoundedRect.iFrameWidth));
	rect.SetSize(TSize(screenRect.Width(), iWindowProperties.iRoundedRect.iFrameWidth));
	
	aRenderer.RenderFillRect(rect, TColor(iWindowProperties.iRoundedRect.iFrameColor));
}


