/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SDL_WINDOW_H
#define __SDL_WINDOW_H

#include "sdlcore.h"
#include "sdlcontainer.h"
#include "sdllabelwidget.h"

namespace sdl
{
	class Window : public Container
	{
	public:
		class Properties
		{
		public:
			Properties(const LabelWidget::Properties& aLabel = LabelWidget::Properties(), int aTitleHeight = -1, const TRoundedRect& aRoundedRect = TRoundedRect())
			: iLabelTitle(aLabel), iTitleHeight(aTitleHeight), iRoundedRect(aRoundedRect) { }
			void OverrideDefaults(const Properties& aOther)
			{
				iLabelTitle.OverrideDefaults(aOther.iLabelTitle);
				OverridePositiveValue(iTitleHeight, aOther.iTitleHeight);
				iRoundedRect.OverrideDefaults(aOther.iRoundedRect);
			}
			LabelWidget::Properties iLabelTitle;
			int						iTitleHeight;
			TRoundedRect			iRoundedRect;
		};
		
		Window() 										{ SetDefaultProperties(); }
		
		void SetTitle(const std::string& aTitle)		{ iLabelTitle.SetText(aTitle); }
		void SetInternalMargin(const TMargin& aMargin)	{ iInternalMargin = aMargin; SizeChanged(GetSize()); }
		void SetAutoCenter(bool aEnable)				{ iAutoCenter = aEnable; }
		void OverrideDefaults(const Properties& aOther) { iWindowProperties.OverrideDefaults(aOther); ApplyProperties(); }
	protected:
		void ApplyProperties()							{ iLabelTitle.OverrideDefaults(iWindowProperties.iLabelTitle); }
		void SetDefaultProperties();
		
		virtual void Render(IRenderer& aRenderer) override;
		virtual void CreateCachedTextures(IRenderer& aRenderer) override;

		virtual void SizeChanged(const TSize& aOldSize) override;
		virtual void PositionChanged(const TPoint& aOldPosition) override;

		void CenterWindow();
		
		LabelWidget	iLabelTitle;
		TMargin		iInternalMargin;
		bool		iAutoCenter = true;
		Properties	iWindowProperties;
	private:
		TRect GetClientArea() const;
	};
}

#endif // __SDL_WINDOW_H