/*******************************************************************
 * 
 *  Author: Peter van Sebille (peter@yipton.net)
 *
 *  (c) Copyright 2016, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************************/

#ifndef __SIMPLE_TEXT_PARSER_H
#define __SIMPLE_TEXT_PARSER_H

#include <string>
#include <vector>
#include <map>
#include <stdio.h>

namespace simpletext 
{
	
typedef std::map<std::string, std::string> TTagAttributes;

class Tag
{
public:
	Tag()			{ }
	void Clear()	{ iName.clear(); iAtributes.clear(); };
	
	std::string		iName;
	TTagAttributes	iAtributes;
};

class Lex;

class IParseConsumer
{
public:
	virtual ~IParseConsumer(){}
	virtual void ConsumeTag(const Tag& aTag) = 0;
	virtual void ConsumeText(const std::string& aText) = 0;
	virtual void Done() = 0;
	virtual void Warning(int aStart, int aLen, const std::string& aError) = 0;
};

class Parser
{
public:
	void Parse(const std::string& aText, IParseConsumer& aParseConsumer);
protected:
	void Parse(Lex& aLex, IParseConsumer& aParseConsumer);
	void ParseTag(Lex& aLex, IParseConsumer& aParseConsumer);
	
	void ResetTag();
	
	Tag			iTag;
	bool		iInTag = false;
	std::string iKey;
	bool 		iParsedEqual;
};

}

#endif // __SIMPLE_TEXT_PARSER_H
